package tests;

import static org.junit.Assert.*;


import java.io.StringReader;
import java.util.*;

import org.junit.Test;

import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import rabinizer.exec.*;
import rabinizer.formulas.*;
import rabinizer.parser.LTLParser;

@RunWith(value = Parameterized.class)
public class Entails {

	private List<String> rawRegularPremises;
	private List<String> rawPromisePremises;
	private String consequence;
	private boolean expected;

	
	public Entails(List<String> rawRegularPremises, List<String> rawPromisePremises, String consequence, boolean expected) {
		this.rawRegularPremises = rawRegularPremises;
		this.rawPromisePremises = rawPromisePremises;
		this.consequence = consequence;
		this.expected = expected;
	}

    @Parameters
	public static Collection<Object[]> data() {
        Object[][] data = new Object[][] {
        		{ Arrays.asList("true"), Arrays.asList("true"), "true", true },
        		{ Arrays.asList("true"), Arrays.asList("true"), "false", false },
        		{ Arrays.asList("true"), Arrays.asList("false"), "true", true },
        		{ Arrays.asList("true"), Arrays.asList("false"), "false", true },
        		{ Arrays.asList("false"), Arrays.asList("true"), "true", true },
        		{ Arrays.asList("false"), Arrays.asList("true"), "false", true },
        		{ Arrays.asList("false"), Arrays.asList("false"), "true", true },
        		{ Arrays.asList("false"), Arrays.asList("false"), "false", true },

        		{ Arrays.asList(), Arrays.asList("true"), "true", true },
        		{ Arrays.asList(), Arrays.asList("true"), "false", false },
        		{ Arrays.asList(), Arrays.asList("false"), "true", true },
        		{ Arrays.asList(), Arrays.asList("false"), "false", true },
        		
        		{ Arrays.asList("true"), Arrays.asList(), "true", true },
        		{ Arrays.asList("true"), Arrays.asList(), "false", false },
        		{ Arrays.asList("false"), Arrays.asList(), "true", true },
        		{ Arrays.asList("false"), Arrays.asList(), "false", true },

        		{ Arrays.asList("a & X b", "true"), Arrays.asList("G F (a & X b)", "F (a & X b)"), "G F (a & X b) & F (a & X b) | (a & X b) | b", true },
        		{ Arrays.asList(), Arrays.asList("F b"), "X F b", true },
            	{ Arrays.asList("X a", "b"), Arrays.asList("G (c | d)"), "X a & X X G (c | d)", true },
            	
                { Arrays.asList(), Arrays.asList("G (c | d)"), "G (c | d)", true },
                { Arrays.asList("a"), Arrays.asList("G (c | d)"), "G (c | d)", true },
                { Arrays.asList("a"), Arrays.asList("G (c | d)"), "a & G (c | d)", true },
                { Arrays.asList(), Arrays.asList("G (c | d)"), "a & G (c | d)", false },
                { Arrays.asList(), Arrays.asList("G (c | d)"), "X a & G (c | d)", false },
                { Arrays.asList("a"), Arrays.asList("G (c | d)"), "X a & G (c | d)", false },
                { Arrays.asList("X a"), Arrays.asList("G (c | d)"), "X a & G (c | d)", true },
            	
                { Arrays.asList("X a", "b"), Arrays.asList("G (c | d)"), "a & G (c | d)", false },
                { Arrays.asList("a | b"), Arrays.asList("F a"), "a | (F a) | b", true },
                { Arrays.asList("a & b"), Arrays.asList("F a"), "a | b", true },
                { Arrays.asList("a", "b"), Arrays.asList("G (c | d)"), "X (a & G(c | d))", false },
        };
        return Arrays.asList(data);
    }

	@Test
	public void pushTest() {
		System.out.println("\n"+this.getClass().getCanonicalName());
		
		
		LTLParser parser = new LTLParser(new StringReader(consequence));
		Formula consf = null;
		try { consf = parser.parse(); }
		catch (Exception e) { assertTrue(false); }

		List<Formula> regularPremises = new ArrayList<Formula>();
		for (String state : rawRegularPremises) {
			parser.ReInit(new StringReader(state));
			try {  regularPremises.add(parser.parsePreviousAtoms()); }
			catch (Exception e) { assertTrue(false); }
		}
		
		ArrayList<Formula> promisePremises = new ArrayList<Formula>();
		for (String state : rawPromisePremises) {
			parser.ReInit(new StringReader(state));
			try {  promisePremises.add(parser.parsePreviousAtoms()); }
			catch (Exception e) { assertTrue(false); }
		}
		
		System.out.println("regular premises: " + regularPremises);
		System.out.println("promise premises: " + promisePremises);
		System.out.println("consequence     : " + consf);
		System.out.println("expected        : " + expected);
		assertTrue(GeneralizedRabinAcceptanceCondition.entails(regularPremises, promisePremises, null, consf) == expected);
		System.out.println("PASSED");
	}
}
